

function GPS_user_pos()

load SAT_INFO

for time_to_check=1:size(SAT_INFO,2)
    if  size(SAT_INFO(time_to_check),2)>0
        upos = Userpos(SAT_INFO(time_to_check).sat_data);
        
        %Estrazione dati di output
        x_ns_ECEF=upos(1); % Posizione x calcolata da noi in ECEF
        y_ns_ECEF=upos(2); % Posizione y calcolata da noi in ECEF
        z_ns_ECEF=upos(3); % Posizione z calcolata da noi in ECEF
        bias_ns=upos(4);
        hns = upos(8);
        LATns = upos(6);
        LNGns = upos(7); % altezza latitudine e longitudine nostre;
        fprintf('Check del time %d \n',time_to_check);
    end

end


end


%% Funzione UPOS [pseudorange+satelliti]-->[posizione utente] 
function [upos] = Userpos(inp)

% sp(1:3, 1:nsat); % satellite position which has the following format
% pr(1:nsat); % is the measured pseudo-range which has the format as
% nn=nsat; % is the number of satellites

[mm nn] = size(inp);
nsat = nn;

sp = inp(1:3,1:nn);
pr = inp(4,:);

% ***** Select initial guessed positions and clock bias *****
x_guess=0; y_guess=0; z_guess=0; bu=0;
gu(1)=x_guess; gu(2)=y_guess; gu(3)=z_guess;

% Calculating rao the pseudo-range as shown in Equation (2.1) the
% clock bias is not included
for j=1:nsat
    rao(j)=((gu(1)-sp(1,j))^2+(gu(2)-sp(2,j))^2+(gu(3)-sp(3,j))^2)^.5;
end

% generate the fourth column of the alpha matrix in Eq. 2.15
alpha(:,4) = ones(nsat,1);
erro = 1;
Omegae_dot = 7.292115147e-5;
c=299792458;

originale_sp = sp;

while erro > .0000000001
    for j = 1:nsat;
        for k = 1:3
            alpha(j,k) = (gu(k)-sp(k,j))/(rao(j)); % find first 3 columns of alpha matrix
        end
    end
    drao = pr - (rao + ones(1,nsat)*bu); % find delta rao includes clock bias
    dl = pinv(alpha)*drao'; 
    bu = bu + dl(4); % new clock bias
    for k = 1:3
        gu(k) = gu(k) + dl(k); %**find new position
    end
    erro = dl(1)^2+dl(2)^2+dl(3)^2; % find error
    for j = 1:nsat;
        rao(j) = ((gu(1)-sp(1,j))^2+(gu(2)-sp(2,j))^2+(gu(3)-sp(3,j))^2)^.5; % find new rao without clock bias
        traveltime = rao(j) / c;
        omegatau = Omegae_dot * traveltime;
        Rotazione = [cos(omegatau) sin(omegatau) 0; -sin(omegatau) cos(omegatau) 0; 0 0 1];
        sp(:,j) = Rotazione * originale_sp(:,j);
    end
end


    % ***** Final result in spherical coordinate system *****

    xuser = gu(1); yuser = gu(2); zuser = gu(3); bias = bu;

    rsp = (xuser^2+yuser^2+zuser^2)^.5; % radius of spherical earth Eq 2.17
    Lc = atan(zuser/(xuser^2+yuser^2)^.5); % latitude of spherical earth Eq 2.18
    lsp = atan(yuser/xuser)*180/pi; % longitude spherical and flat earth Eq 2.19

    % ***** Converting to practical earth shape *****
    e = 1/298.257223563;
    Ltemp = Lc;
    erro1 = 1;

    while erro1>1e-12; % calculating latitude by Eq. 2.51
        L = Lc+e*sin(2*Ltemp);
        erro1 = abs(Ltemp-L);
        Ltemp = L;
    end
    Lflp = L*180/pi; % latitude of flat earth
    re = 6378137;
    h = rsp-re* (1-e*(sin(L)^2)); % altitude of flat earth
    
    upos = [xuser yuser zuser bias rsp Lflp lsp h]';

end

%% Conversione ECEF ->latlng
function [Lflp lsp h]=ECEF2latlng(xuser,yuser,zuser)

    rsp = (xuser^2+yuser^2+zuser^2)^.5; % radius of spherical earth Eq 2.17
    Lc = atan(zuser/(xuser^2+yuser^2)^.5); % latitude of spherical earth Eq 2.18
    lsp = atan(yuser/xuser)*180/pi; % longitude spherical and flat earth Eq 2.19

    % ***** Converting to practical earth shape *****
    e = 1/298.257223563;
    Ltemp = Lc;
    erro1 = 1;

    while erro1>1e-12; % calculating latitude by Eq. 2.51
        L = Lc+e*sin(2*Ltemp);
        erro1 = abs(Ltemp-L);
        Ltemp = L;
    end
    Lflp = L*180/pi; % latitude of flat earth
    re = 6378137;
    h = rsp-re* (1-e*(sin(L)^2)); % altitude of flat earth

end




